@@ -1,11 +1,13 @@
 import logging
+from tokenizers import Tokenizer  # type: ignore
 from types import TracebackType
 from typing import Any, List, Mapping, Optional, Dict, Sequence, Tuple, Type, Union
 import warnings
-
 import aiohttp
+from aiohttp import ClientResponse
 from aiohttp_retry import RetryClient, ExponentialRetry
 import requests
+from requests import Response
 from requests.adapters import HTTPAdapter
 from requests.structures import CaseInsensitiveDict
 from urllib3.util.retry import Retry
@@ -14,7 +16,7 @@ import aleph_alpha_client
 from aleph_alpha_client.document import Document
 from aleph_alpha_client.explanation import ExplanationRequest, ExplanationResponse
 from aleph_alpha_client.image import Image
-from aleph_alpha_client.prompt import _to_prompt_item, _to_serializable_prompt
+from aleph_alpha_client.prompt import _to_json, _to_serializable_prompt
 from aleph_alpha_client.summarization import SummarizationRequest, SummarizationResponse
 from aleph_alpha_client.qa import QaRequest, QaResponse
 from aleph_alpha_client.completion import CompletionRequest, CompletionResponse
@@ -263,6 +265,8 @@ class AlephAlphaClient:
         completion_bias_inclusion_first_token_only: bool = False,
         completion_bias_exclusion: Optional[Sequence[str]] = None,
         completion_bias_exclusion_first_token_only: bool = False,
+        contextual_control_threshold: Optional[float] = None,
+        control_log_additive: Optional[bool] = True,
     ):
         """Generates samples from a prompt.
 
@@ -403,6 +407,18 @@ class AlephAlphaClient:
 
             completion_bias_exclusion_first_token_only (bool, default False)
                 Only consider the first token for the completion_bias_exclusion
+
+            contextual_control_threshold (float, default None)
+                If set to None, attention control parameters only apply to those tokens that have
+                explicitly been set in the request.
+                If set to a non-None value, we apply the control parameters to similar tokens as well.
+                Controls that have been applied to one token will then be applied to all other tokens
+                that have at least the similarity score defined by this parameter.
+                The similarity score is the cosine similarity of token embeddings.
+
+            control_log_additive (bool, default True)
+                True: apply control by adding the log(control_factor) to attention scores.
+                False: apply control by (attention_scores - - attention_scores.min(-1)) * control_factor
         """
 
         payload = {
@@ -436,6 +452,8 @@ class AlephAlphaClient:
             "completion_bias_inclusion_first_token_only": completion_bias_inclusion_first_token_only,
             "completion_bias_exclusion": completion_bias_exclusion or [],
             "completion_bias_exclusion_first_token_only": completion_bias_exclusion_first_token_only,
+            "contextual_control_threshold": contextual_control_threshold,
+            "control_log_additive": control_log_additive,
         }
 
         if hosting is not None:
@@ -465,6 +483,8 @@ class AlephAlphaClient:
         tokens: Optional[bool] = False,
         type: Optional[str] = None,
         normalize: bool = False,
+        contextual_control_threshold: Optional[float] = None,
+        control_log_additive: Optional[bool] = True,
     ):
         """
         Embeds a text and returns vectors that can be used for downstream tasks (e.g. semantic similarity) and models (e.g. classifiers).
@@ -510,6 +530,18 @@ class AlephAlphaClient:
 
             normalize
                 Return normalized embeddings. This can be used to save on additional compute when applying a cosine similarity metric.
+
+            contextual_control_threshold (float, default None)
+                If set to None, attention control parameters only apply to those tokens that have
+                explicitly been set in the request.
+                If set to a non-None value, we apply the control parameters to similar tokens as well.
+                Controls that have been applied to one token will then be applied to all other tokens
+                that have at least the similarity score defined by this parameter.
+                The similarity score is the cosine similarity of token embeddings.
+
+            control_log_additive (bool, default True)
+                True: apply control by adding the log(control_factor) to attention scores.
+                False: apply control by (attention_scores - - attention_scores.min(-1)) * control_factor
         """
 
         serializable_prompt = _to_serializable_prompt(
@@ -527,6 +559,8 @@ class AlephAlphaClient:
             "pooling": pooling,
             "type": type,
             "normalize": normalize,
+            "contextual_control_threshold": contextual_control_threshold,
+            "control_log_additive": control_log_additive,
         }
 
         if hosting is not None:
@@ -545,7 +579,6 @@ class AlephAlphaClient:
         model: str,
         request: SemanticEmbeddingRequest,
         hosting: Optional[str] = None,
-        normalize: bool = False,
     ):
         """
         Embeds a text and returns vectors that can be used for downstream tasks (e.g. semantic similarity) and models (e.g. classifiers).
@@ -566,9 +599,6 @@ class AlephAlphaClient:
 
             request (SemanticEmbeddingRequest, required)
                 NamedTuple containing all necessary request parameters.
-
-            normalize
-                Return normalized embeddings. This can be used to save on additional compute when applying a cosine similarity metric.
         """
 
         serializable_prompt = request.prompt.to_json()
@@ -578,7 +608,9 @@ class AlephAlphaClient:
             "prompt": serializable_prompt,
             "representation": request.representation.value,
             "compress_to_size": request.compress_to_size,
-            "normalize": normalize,
+            "normalize": request.normalize,
+            "contextual_control_threshold": request.contextual_control_threshold,
+            "control_log_additive": request.control_log_additive,
         }
 
         if hosting is not None:
@@ -598,6 +630,8 @@ class AlephAlphaClient:
         completion_expected,
         hosting: Optional[str] = None,
         prompt: Union[str, List[Union[str, Image]]] = "",
+        contextual_control_threshold: Optional[float] = None,
+        control_log_additive: Optional[bool] = True,
     ):
         """
         Evaluates the model's likelihood to produce a completion given a prompt.
@@ -621,6 +655,18 @@ class AlephAlphaClient:
 
             prompt (str, optional, default ""):
                 The text to be completed. Unconditional completion can be used with an empty string (default). The prompt may contain a zero shot or few shot task.
+
+            contextual_control_threshold (float, default None)
+                If set to None, attention control parameters only apply to those tokens that have
+                explicitly been set in the request.
+                If set to a non-None value, we apply the control parameters to similar tokens as well.
+                Controls that have been applied to one token will then be applied to all other tokens
+                that have at least the similarity score defined by this parameter.
+                The similarity score is the cosine similarity of token embeddings.
+
+            control_log_additive (bool, default True)
+                True: apply control by adding the log(control_factor) to attention scores.
+                False: apply control by (attention_scores - - attention_scores.min(-1)) * control_factor
         """
 
         serializable_prompt = _to_serializable_prompt(prompt=prompt)
@@ -629,6 +675,8 @@ class AlephAlphaClient:
             "model": model,
             "prompt": serializable_prompt,
             "completion_expected": completion_expected,
+            "contextual_control_threshold": contextual_control_threshold,
+            "control_log_additive": control_log_additive,
         }
 
         if hosting is not None:
@@ -777,7 +825,7 @@ class AlephAlphaClient:
     ):
         body = {
             "model": model,
-            "prompt": [_to_prompt_item(item) for item in request.prompt.items],
+            "prompt": [_to_json(item) for item in request.prompt.items],
             "target": request.target,
             "suppression_factor": request.suppression_factor,
             "conceptual_suppression_threshold": request.conceptual_suppression_threshold,
@@ -894,10 +942,13 @@ class Client:
 
     def get_version(self) -> str:
         """Gets version of the AlephAlpha HTTP API."""
-        response = self.session.get(self.host + "version")
+        return self._get_request("version").text
+
+    def _get_request(self, endpoint: str) -> Response:
+        response = self.session.get(self.host + endpoint)
         if not response.ok:
             _raise_for_status(response.status_code, response.text)
-        return response.text
+        return response
 
     def _post_request(
         self,
@@ -938,6 +989,15 @@ class Client:
             json_body["hosting"] = self.hosting
         return json_body
 
+    def models(self) -> List[Mapping[str, Any]]:
+        """
+        Queries all models which are currently available.
+
+        For documentation of the response, see https://docs.aleph-alpha.com/api/available-models/
+        """
+        response = self._get_request("models_available")
+        return response.json()
+
     def complete(
         self,
         request: CompletionRequest,
@@ -955,7 +1015,7 @@ class Client:
 
         Examples:
             >>> # create a prompt
-            >>> prompt = Prompt("An apple a day, ")
+            >>> prompt = Prompt.from_text("An apple a day, ")
             >>>
             >>> # create a completion request
             >>> request = CompletionRequest(
@@ -1121,7 +1181,7 @@ class Client:
 
         Examples:
             >>> request = EvaluationRequest(
-                    prompt=Prompt.from_text("hello"), completion_expected="world"
+                    prompt=Prompt.from_text("hello"), completion_expected=" world"
                 )
             >>> response = client.evaluate(request, model=model_name)
         """
@@ -1219,6 +1279,15 @@ class Client:
         response = self._post_request("search", request, None)
         return SearchResponse.from_json(response)
 
+    def tokenizer(self, model: str) -> Tokenizer:
+        """Returns a Tokenizer instance with the settings that were used to train the model.
+
+        Examples:
+            >>> tokenizer = client.tokenizer(model="luminous-extended")
+            >>> tokenized_prompt = tokenizer.encode("Hello world")
+        """
+        return Tokenizer.from_str(self._get_request(f"models/{model}/tokenizer").text)
+
 
 class AsyncClient:
     """
@@ -1324,13 +1393,26 @@ class AsyncClient:
 
     async def get_version(self) -> str:
         """Gets version of the AlephAlpha HTTP API."""
+        return await self._get_request_text("version")
+
+    async def _get_request_text(self, endpoint: str) -> str:
         async with self.session.get(
-            self.host + "version",
+            self.host + endpoint,
         ) as response:
             if not response.ok:
                 _raise_for_status(response.status, await response.text())
             return await response.text()
 
+    async def _get_request_json(
+        self, endpoint: str
+    ) -> Union[List[Mapping[str, Any]], Mapping[str, Any]]:
+        async with self.session.get(
+            self.host + endpoint,
+        ) as response:
+            if not response.ok:
+                _raise_for_status(response.status, await response.text())
+            return await response.json()
+
     async def _post_request(
         self,
         endpoint: str,
@@ -1367,6 +1449,14 @@ class AsyncClient:
             json_body["hosting"] = self.hosting
         return json_body
 
+    async def models(self) -> List[Mapping[str, Any]]:
+        """
+        Queries all models which are currently available.
+
+        For documentation of the response, see https://docs.aleph-alpha.com/api/available-models/
+        """
+        return await self._get_request_json("models_available")  # type: ignore
+
     async def complete(
         self,
         request: CompletionRequest,
@@ -1384,7 +1474,7 @@ class AsyncClient:
 
         Examples:
             >>> # create a prompt
-            >>> prompt = Prompt("An apple a day, ")
+            >>> prompt = Prompt.from_text("An apple a day, ")
             >>>
             >>> # create a completion request
             >>> request = CompletionRequest(
@@ -1549,7 +1639,7 @@ class AsyncClient:
 
         Examples:
             >>> request = EvaluationRequest(
-                    prompt=Prompt.from_text("hello"), completion_expected="world"
+                    prompt=Prompt.from_text("hello"), completion_expected=" world"
                 )
             >>> response = await client.evaluate(request, model=model_name)
         """
@@ -1646,3 +1736,13 @@ class AsyncClient:
         """
         response = await self._post_request("search", request, None)
         return SearchResponse.from_json(response)
+
+    async def tokenizer(self, model: str) -> Tokenizer:
+        """Returns a Tokenizer instance with the settings that were used to train the model.
+
+        Examples:
+            >>> tokenizer = await client.tokenizer(model="luminous-extended")
+            >>> tokenized_prompt = tokenizer.encode("Hello world")
+        """
+        response = await self._get_request_text(f"models/{model}/tokenizer")
+        return Tokenizer.from_str(response)
